import json
import requests

class Amplitude:

    def __init__(self, api_key):
        self.url = "https://api.amplitude.com/identify"
        self.api_key = api_key
    
    @staticmethod
    def _parse(key):
        return key.split(":")

    def post(self, key, value):

        _, ab_test_id, user_id = self._parse(key)

        user_properties = {
            f"abTest.{ab_test_id}": value.get("ab_test_group")
        }

        data = {
            "api_key": self.api_key,
            "identification": json.dumps(
                [
                    {
                        "user_id": user_id,
                        "user_properties": user_properties
                    }
                ]
            )
        }

        response = requests.post(self.url, data=data)


from functools import wraps

def ab(func):
    
    @wraps(func)
    async def wrapper(*args, **kwargs):
        
        request = kwargs.get("request")
        background_tasks = kwargs.get("background_tasks")

        body = await request.json()
        user_id = body.get("user_id")
        
        choixpeau = request.app.state.choixpeau

        ab_tests = choixpeau.get(user_id)

        for key, value, _  in ab_tests:
            if key:
                background_tasks.add_task(choixpeau.store, key, value)

        setattr(
            request.state, 
            "user", 
            [
                {
                    "user_id": user_id, 
                    "ab_test_id": ab_test_id, 
                    **value
                } 
                for _, value, ab_test_id in ab_tests
            ]
        )         
        
        return await func(*args, **kwargs)

    return wrapper


import redis
import random
import datetime
from .amplitude import Amplitude

class Choixpeau:

    def __init__(
        self, 
        redis_config, 
        ab_test_ids=[], 
        buckets=["A", "B"], 
        amplitude_api_key=None
    ):
        self._r = redis.Redis(**redis_config, decode_responses=True)
        self.ab_test_ids = ab_test_ids
        self.buckets = buckets
        
        if amplitude_api_key:
            self.amplitude = Amplitude(api_key=amplitude_api_key) 

    def _draw(self):
        return {
            "created_at": datetime.datetime.today().strftime("%Y-%m-%d"), 
            "ab_test_group": random.choice(self.buckets)
        }      

    def _get(self, ab_test_id, user_id):
        
        key = f"ab:{ab_test_id}:{user_id}"
        value = self._r.hgetall(key)

        if value:
            return None, value, ab_test_id 
        else:
            return key, self._draw(), ab_test_id
    
    def get(self, user_id):
        return [self._get(ab_test_id, user_id) for ab_test_id in self.ab_test_ids]

    def store(self, key, value):
        self._r.hset(key, mapping=value)
        if hasattr(self, "amplitude"):
            self.amplitude.post(key, value)